﻿namespace TabsStudioSQLComment
{
    public class SQLComment : TabsStudioExt.ITabsStudioAddin, TabsStudioExt.IConfigurable, EnvDTE.IDTCommandTarget
    {
        public SQLComment()
        {
            commentTabExtensionMenuItem = new System.Windows.Controls.MenuItem();
            commentTabExtensionMenuItem.Header = "Set SQL comment...";
            commentTabExtensionMenuItem.Click += CommentTabExtension;

            commentTabMenuItem = new System.Windows.Controls.MenuItem();
            commentTabMenuItem.Header = "Set SQL comment...";
            commentTabMenuItem.Click += CommentTab;
        }

        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte_)
        {
            engine = engine_;
            dte = dte_;
            engine.TabCreated += OnTabCreated;
            engine.TabExtensionCreated += OnTabExtensionCreated;
            engine.TabExtensionDestroyed += OnTabExtensionDestroyed;
            engine.OpeningTabExtensionContextMenu += OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu += OpeningTabContextMenu;

            storage = new OptionsStorage(engine.GetAddinSettingsDirectory());
            options = storage.Load();

            tabManager = new TabManager(options);

            commandsImpl = new CommandsImpl(tabManager, engine, dte);
            commands = new Commands(engine, dte, commandsImpl);
        }

        public void OnDisconnection()
        {
            engine.TabCreated -= OnTabCreated;
            engine.TabExtensionCreated -= OnTabExtensionCreated;
            engine.TabExtensionDestroyed -= OnTabExtensionDestroyed;
            engine.OpeningTabExtensionContextMenu -= OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu -= OpeningTabContextMenu;
        }

        public void QueryStatus(string CmdName, EnvDTE.vsCommandStatusTextWanted NeededText,
               ref EnvDTE.vsCommandStatus StatusOption, ref System.Object CommandText)
        {
            commands.QueryStatus(CmdName, NeededText, ref StatusOption, ref CommandText);
        }

        public void Exec(string CmdName, EnvDTE.vsCommandExecOption ExecuteOption, ref System.Object VariantIn,
            ref System.Object VariantOut, ref bool Handled)
        {
            commands.Exec(CmdName, ExecuteOption, ref VariantIn, ref VariantOut, ref Handled);
        }

        public void Configure(System.Windows.Window parent)
        {
            OptionsDialog dialog = new OptionsDialog(options);
            dialog.Owner = parent;
            dialog.WindowStartupLocation = System.Windows.WindowStartupLocation.CenterOwner;
            if (dialog.ShowDialog().GetValueOrDefault())
                storage.Save(options);
        }

        ///

        private void OpeningTabExtensionContextMenu(object sender,
            TabsStudioExt.OpeningTabExtensionContextMenuEventArgs e)
        {
            storedTab = e.Tab;
            storedTabExtension = e.TabExtension;
            PlaceMenuItem(commentTabExtensionMenuItem, e.ContextMenu);
        }

        private void OpeningTabContextMenu(object sender, TabsStudioExt.OpeningTabContextMenuEventArgs e)
        {
            storedTab = e.Tab;
            PlaceMenuItem(commentTabMenuItem, e.ContextMenu);
        }

        private void CommentTabExtension(object sender, System.Windows.RoutedEventArgs e)
        {
            commandsImpl.CommentTabExtension(storedTab, storedTabExtension);
        }

        private void CommentTab(object sender, System.Windows.RoutedEventArgs e)
        {
            commandsImpl.CommentTab(storedTab);
        }

        private void PlaceMenuItem(System.Windows.Controls.MenuItem menuItem,
            System.Windows.Controls.ContextMenu menu)
        {
            foreach (object item in menu.Items)
            {
                if (item is System.Windows.Controls.Control)
                {
                    if ((item as System.Windows.Controls.Control).Name.Equals("OpenContainingFolder"))
                    {
                        menu.Items.Insert(menu.Items.IndexOf(item) + 1, menuItem);
                        break;
                    }
                }
            }
        }

        ///

        private void OnTabCreated(object sender, TabsStudioExt.TabEventArgs e)
        {
            tabManager.OnTabCreated(e);
        }

        private void OnTabExtensionCreated(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            tabManager.OnTabExtensionCreated(e);
        }

        private void OnTabExtensionDestroyed(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            tabManager.OnTabExtensionDestroyed(e);
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private EnvDTE80.DTE2 dte;
        private OptionsStorage storage;
        private Options options;
        private TabsStudioExt.TabExtension storedTabExtension;
        private System.Windows.Controls.MenuItem commentTabExtensionMenuItem;
        private TabsStudioExt.Tab storedTab;
        private System.Windows.Controls.MenuItem commentTabMenuItem;
        private TabManager tabManager;
        private CommandsImpl commandsImpl;
        private Commands commands;
    }
}
